from griff.services.abstract_service import AbstractService


class DictService(AbstractService):
    @staticmethod
    def keep_keys(d: dict, keys: list):
        """
        Remove key not in keys from dictionary

        Args:
            d(dict): dictionary to clean
            keys(list): dictionary keys to keep

        Returns:
            dict: cleaned dictionary
        """
        return {k: v for k, v in d.items() if k in keys}

    @staticmethod
    def remove_keys(d: dict, keys: list):
        """
        Remove key in keys from dictionary

        Args:
            d(dict): dictionary to clean
            keys(list): dictionary keys to remove

        Returns:
            dict: cleaned dictionary
        """
        return {k: v for k, v in d.items() if k not in keys}

    @staticmethod
    def get_last_entry_value(d: dict):
        """
        get last entry value

        Args:
            d(dict): dictonnary

        Returns:
            mixed
        """
        return d[list(d.keys())[-1]]

    @staticmethod
    def remap_keys(d: dict, remap_keys: dict) -> dict:
        """remap a dictionnary keys and keep only key found in remap_keys"""
        remaped = {}
        for old, new in remap_keys.items():
            remaped[new] = d[old]
        return remaped



import pytest
from typing import Type

from griff.runtime.factories.abstract_runtime_factory import Injectables
from griff.test_utils.testcases.persistence_testcase import PersistenceTestCase

from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_repository import {{cookiecutter.aggregate | pascal_case}}Repository
from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_persistence import {{cookiecutter.aggregate | pascal_case}}Persistence
from {{ cookiecutter.context | snake_case }}._common.test_utils.{{ cookiecutter.context | snake_case }}_dtf import {{ cookiecutter.context | pascal_case }}Dtf
from {{ cookiecutter.context | snake_case }}.entry_point import {{ cookiecutter.context | pascal_case }}EntryPoint


@pytest.mark.skip(reason="dès qu'il faut tester une requete")
class Test{{cookiecutter.aggregate | pascal_case}}Persistence(PersistenceTestCase):
    @classmethod
    def entry_point_class(cls) -> Type[{{ cookiecutter.context | pascal_case }}EntryPoint]:
        return {{ cookiecutter.context | pascal_case }}EntryPoint

    @classmethod
    def with_injectables(cls) -> Injectables | None:
        return {}

    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.dtf = {{ cookiecutter.context | pascal_case }}Dtf()
        cls.persistence = cls.get_injected({{cookiecutter.aggregate | pascal_case}}Persistence)
        cls.repository = cls.get_injected({{cookiecutter.aggregate | pascal_case}}Repository)
        cls.{{ cookiecutter.aggregate | snake_case }}s = [
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
        ]

    def setup_method(self):
        super().setup_method()

    async def async_setup(self):
        await super().async_setup()
        await self.dtf.persist(self.repository, self.{{ cookiecutter.aggregate | snake_case }}s)

    """
    nom_requete
    """

    async def test_nom_requete_succes(self):
        actual = await self.persistence.run_query("nom_requete")
        self.assert_equals_resultset(actual)



from typing import Type

from griff.runtime.factories.abstract_runtime_factory import Injectables
from griff.test_utils.pytest_runtime_factory import PytestRuntimeFactory
from griff.test_utils.testcases.command_handler_testcase import CommandHandlerTestCase

from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_command import ({{ cookiecutter.command | pascal_case }}Handler, {{ cookiecutter.command | pascal_case }}Command)
from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_repository import {{cookiecutter.aggregate | pascal_case}}Repository
from {{ cookiecutter.context | snake_case }}._common.test_utils.{{ cookiecutter.context | snake_case }}_dtf import {{ cookiecutter.context | pascal_case }}Dtf

class Test{{ cookiecutter.command | pascal_case }}Command(CommandHandlerTestCase):
    @classmethod
    def runtime_factory(cls) -> PytestRuntimeFactory:
        return super().runtime_factory().with_context_fake_persistence("{{ cookiecutter.context | snake_case }}")

    @property
    def handler_class(self) -> Type[{{ cookiecutter.command | pascal_case }}Handler]:
        return {{ cookiecutter.command | pascal_case }}Handler

    @classmethod
    def with_injectables(cls) -> Injectables | None:
        return None

    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.dtf = cls.get_injected({{cookiecutter.context | pascal_case}}Dtf)
        cls.dtf.reset_sequence_id(99990)
        cls.repository = cls.get_injected({{cookiecutter.aggregate | pascal_case}}Repository)
        {{cookiecutter.aggregate | snake_case}} = cls.dtf.creer_{{cookiecutter.aggregate | snake_case}}()
        cls.command = {{ cookiecutter.command | pascal_case }}Command(**{{cookiecutter.aggregate | snake_case}}.model_dump())

    def setup_method(self):
        super().setup_method()
        self.dtf.reset(1)

    async def async_setup(self):
        await super().async_setup()
        self.repository.reset()
        await self.dtf.persist_created("{{cookiecutter.aggregate | snake_case}}")

    """
    handle
    """

    async def test_{{ cookiecutter.command | snake_case }}_avec_un_pb_echoue(self):
        command = self.command.model_copy(update={"un_attr": "une valeur qui fait échoué"})
        response = await self.handler.handle(command)
        self.assert_equals_resultset(response.model_dump())

    async def test_{{ cookiecutter.command | snake_case }}_reussi(self):
        response = await self.handler.handle(self.command)
        self.assert_equals_resultset(
            self.prepare_success_resultset(response, self.repository)
        )




import typer
from griff.appli.command.command_bus import CommandBus
from griff.infra.cli.cli_controller import CliController
from griff.infra.cli.register_cli_command import register_cli_command
from griff.infra.cli.typer_print import TyperPrint
from griff.utils.async_utils import AsyncUtils
from injector import inject

from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_command import {{ cookiecutter.command | pascal_case }}Command


class {{ cookiecutter.command | pascal_case }}CliController(CliController):
    @inject
    def __init__(self, command_bus: CommandBus):
        super().__init__()
        self._command_bus = command_bus

    def get_command_name(self):
        return "{{ cookiecutter.aggregate | snake_case }}"

    async def _dispatch_command(self,  params1: str):
        command = {{ cookiecutter.command | pascal_case }}Command( params1=params1)
        response = await self._command_bus.dispatch(command)
        if response.is_success:
            return response
        TyperPrint.error(f"Echec action : {response.content.message}")
        raise typer.Exit()


    @register_cli_command(name="{{ cookiecutter.command | pascal_case }}")
    def {{ cookiecutter.command | pascal_case }}(self, params1: str):
        p = {"params1": params1}
        response = AsyncUtils.async_to_sync(self._dispatch_command, p)
        if response.is_success:
            TyperPrint.success("Action terminée")



from pathlib import Path

from cookiecutter.main import cookiecutter

from griff.infra.cli.cli_controller import CliController
from griff.infra.cli.register_cli_command import register_cli_command


class GriffCliController(CliController):
    def __init__(self):
        super().__init__()
        self.root_tpl_path = Path(__file__).parent.joinpath("tpl")

    def get_command_name(self) -> str:
        return "griff"

    @register_cli_command(name="start-context")
    def start_context(self, context_name: str, output_dir=".", no_input=False):
        tpl_path = self.root_tpl_path.joinpath("context")
        cookiecutter(
            str(tpl_path),
            output_dir=output_dir,
            overwrite_if_exists=True,
            skip_if_file_exists=True,
            no_input=no_input,
            extra_context={"context": context_name},
        )
        return None

    @register_cli_command(name="start-agg")
    def start_agg(
        self,
        context_name: str,
        aggregate_name: str,
        output_dir=".",
        no_input=False,
    ):
        tpl_path = self.root_tpl_path.joinpath("aggregate")
        cookiecutter(
            str(tpl_path),
            output_dir=output_dir,
            no_input=no_input,
            overwrite_if_exists=True,
            skip_if_file_exists=True,
            extra_context={
                "context": context_name,
                "aggregate": aggregate_name,
            },
        )
        return None

    @register_cli_command(name="start-command")
    def start_command(
        self,
        context_name: str,
        aggregate_name: str,
        command_name: str,
        output_dir=".",
        no_input=False,
    ):
        tpl_path = self.root_tpl_path.joinpath("command")
        cookiecutter(
            str(tpl_path),
            output_dir=output_dir,
            no_input=no_input,
            overwrite_if_exists=True,
            skip_if_file_exists=True,
            extra_context={
                "context": context_name,
                "aggregate": aggregate_name,
                "command": command_name,
            },
        )
        return None



from griff.domain.common_types import DTO
from griff.services.service_locator.service_locator import ServiceLocator
from griff.services.uniqid.uniqid_service import UniqIdService

from {{ cookiecutter.context | snake_case }}._common.domain.{{ cookiecutter.aggregate | snake_case }} import {{cookiecutter.aggregate | pascal_case}}, Base{{cookiecutter.aggregate | pascal_case}}

# todo : à renommer
class Action{{cookiecutter.aggregate | pascal_case}}(Base{{cookiecutter.aggregate | pascal_case}}, DTO):
    ...

def {{ cookiecutter.command | snake_case }}({{ cookiecutter.aggregate | snake_case }}_action: Action{{cookiecutter.aggregate | pascal_case}}) -> {{cookiecutter.aggregate | pascal_case}}:
    entity_id = ServiceLocator.get(UniqIdService).get()
    return {{cookiecutter.aggregate | pascal_case}}(entity_id=entity_id, **{{ cookiecutter.aggregate | snake_case }}_action.model_dump())



import copy
from abc import ABC
from typing import Self

from injector import inject

from griff.infra.persistence.persistence import Persistence, QueryRowResult
from griff.services.json.json_service import JsonService


class SerializedPersistence(Persistence, ABC):
    @inject
    def __init__(self, json_service: JsonService):
        self.json_service = json_service
        self._metadata_fields = ["created_at", "updated_at"]
        self._set_excluded_fields(
            [*self._metadata_fields, *self._serialize_excluded_fields]
        )

    def add_excluded_field(self, field: str) -> Self:
        # à utiliser uniquement dans un repository
        if field not in self._excluded_fields:
            self._excluded_fields.append(field)
        return self

    @property
    def _serialized_persistence_fieldname(self):
        return "serialized"

    @property
    def _serialize_excluded_fields(self):
        return []

    def _set_excluded_fields(self, fields: list[str]) -> None:
        self._excluded_fields = list(dict.fromkeys(fields))

    def _prepare_to_save(self, data: dict) -> dict:
        data_to_serialize = {
            k: v for k, v in data.items() if not self._is_field_serialize_excluded(k)
        }
        attr_serialized = data_to_serialize.keys()
        pk = self._get_pk_from_data(data)
        return {
            **pk,
            self._serialized_persistence_fieldname: self.json_service.dump(
                data_to_serialize
            ),
            **{k: v for k, v in data.items() if k not in attr_serialized},
        }

    def _prepare_row_result(self, result: QueryRowResult) -> QueryRowResult:
        result_copy = copy.deepcopy(result)
        serialized = result_copy.pop(self._serialized_persistence_fieldname)
        return {**result_copy, **self.json_service.load_from_str(serialized)}

    def _is_field_serialize_excluded(self, field_name) -> bool:
        return field_name in self._excluded_fields



from griff.context_entry_point import ContextEntryPoint
from injector import inject

from {{ cookiecutter.context | snake_case }}._common.{{ cookiecutter.context | snake_case }}_cli_router import {{ cookiecutter.context | pascal_case }}CliRouter
from {{ cookiecutter.context | snake_case }}._common.{{ cookiecutter.context | snake_case }}_router import {{ cookiecutter.context | pascal_case }}ApiRouter

class {{ cookiecutter.context | pascal_case }}EntryPoint(ContextEntryPoint[{{ cookiecutter.context | pascal_case }}ApiRouter, {{ cookiecutter.context | pascal_case }}CliRouter]):
    @inject
    def __init__(self, api_router: {{ cookiecutter.context | pascal_case }}ApiRouter, cli_router: {{ cookiecutter.context | pascal_case }}CliRouter):
        super().__init__(api_router, cli_router)

    @staticmethod
    def context_name() -> str:
        return "{{ cookiecutter.context | snake_case }}"



from typing import Type

from injector import inject

from griff.infra.repository.repository import Repository, FakeRepositoryMixin
from griff.services.date.date_service import DateService
from griff.infra.repository.serialized_repository import SerializedRepository

from {{ cookiecutter.context | snake_case }}._common.domain.{{ cookiecutter.aggregate | snake_case }} import {{cookiecutter.aggregate | pascal_case}}
from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_persistence import {{cookiecutter.aggregate | pascal_case}}Persistence

class {{cookiecutter.aggregate | pascal_case}}Repository(SerializedRepository[{{cookiecutter.aggregate | pascal_case}}]):
    @inject
    def __init__(
        self, persistence: {{cookiecutter.aggregate | pascal_case}}Persistence,
        date_service: DateService
    ):
        super().__init__(persistence, date_service)

    @staticmethod
    def _aggregate_class(raw_data: dict | None = None) -> Type[{{cookiecutter.aggregate | pascal_case}}]:
        return {{cookiecutter.aggregate | pascal_case}}

######
# OU (supprimer l'implémentation inutile)
######
class {{cookiecutter.aggregate | pascal_case}}Repository(Repository[{{cookiecutter.aggregate | pascal_case}}]):
    @inject
    def __init__(
        self, persistence: {{cookiecutter.aggregate | pascal_case}}Persistence, date_service: DateService
    ):
        super().__init__(persistence, date_service)

    @staticmethod
    def _aggregate_class(raw_data: dict | None = None) -> Type[{{cookiecutter.aggregate | pascal_case}}]:  # pragma: no cover
        return {{cookiecutter.aggregate | pascal_case}}






class {{cookiecutter.aggregate | pascal_case}}FakeRepository(FakeRepositoryMixin, {{cookiecutter.aggregate | pascal_case}}Repository):
    ...



from abc import ABC
from typing import TypeVar, Generic

from griff.domain.common_types import TenantEntity, EntityId, TenantId
from griff.infra.repository.base_repository import BaseRepository

A = TypeVar("A", bound=TenantEntity)


# noinspection PyMethodOverriding
class TenantRepository(Generic[A], BaseRepository[A], ABC):
    async def get_by_id(self, entity_id: EntityId, tenant_id: TenantId) -> A | None:
        return await self._check_by_id(entity_id=entity_id, tenant_id=tenant_id)

    async def list_all(self, tenant_id: TenantId) -> list[A]:
        return await self._list_all(tenant_id=tenant_id)

    async def run_query(self, query_name: str, tenant_id: TenantId, **query_params):
        query_params["tenant_id"] = tenant_id
        return await self._run_query(query_name=query_name, **query_params)

    async def check_by_aggregate(self, aggregate: A) -> A | None:
        return await self.get_by_id(
            entity_id=aggregate.entity_id, tenant_id=aggregate.tenant_id
        )

    async def _get_persistence_by_aggregate(self, aggregate: A) -> dict | None:
        return await self._get_persistence_by_id(
            entity_id=aggregate.entity_id, tenant_id=aggregate.tenant_id
        )



from griff.appli.event.event import Event

from {{ cookiecutter.context | snake_case }}._common.domain.{{ cookiecutter.aggregate | snake_case }} import {{cookiecutter.aggregate | pascal_case}}

# todo: renommer correctement l'evt
class {{ cookiecutter.CommandEvent }}(Event):
    payload: {{cookiecutter.aggregate | pascal_case}}



import copy
from typing import Dict, List, Callable, Any

from griff.infra.persistence.persistence import (
    Persistence,
    QueryRowResult,
    QueryResult,
    QueryRowResults,
    Id,
    IdName,
    IdValue,
)
from griff.infra.persistence.serialized_persistence import SerializedPersistence
from griff.services.json.json_service import JsonService

QueryName = str
QueryCallable = Callable[..., QueryResult]


def _has_filters_match(data: Dict[str, Dict], filters: Dict[str, Any]) -> bool:
    for attr, value in filters.items():
        if data.get(attr) != value:
            return False
    return True


class DictPersistence(Persistence):
    _pk: List[IdName] = ["entity_id"]

    def __init__(self, initial_data: List[Dict] | None = None):
        self._internal_storage: List = []
        self.reset(initial_data)

    async def _insert(self, data: dict) -> None:
        idx = self._search_idx_from_data(data)
        if idx is None:
            self._internal_storage.append(data)
            return None
        pk = self._get_pk_from_data(data)
        raise ValueError(f"id '{self._pk_str(pk)}' already exists")

    async def _update(self, data: dict) -> None:
        idx = self._search_idx_from_data(data)
        if idx is not None:
            self._internal_storage[idx] = data
            return None
        pk = self._get_pk_from_data(data)
        raise ValueError(f"id '{self._pk_str(pk)}' does not exists")

    async def _delete(self, persistence_id: Id) -> None:
        idx = self._search_idx_from_persistence_id(persistence_id)
        if idx is not None:
            self._internal_storage.pop(idx)
            return None
        pk = self._get_pk_from_persistence_id(persistence_id)
        raise ValueError(f"id '{self._pk_str(pk)}' does not exists")

    async def _get_by_id(self, persistence_id: Id) -> QueryRowResult:
        idx = self._search_idx_from_persistence_id(persistence_id)
        if idx is not None:
            return copy.deepcopy(self._internal_storage[idx])
        pk = self._get_pk_from_persistence_id(persistence_id)
        raise ValueError(f"id '{self._pk_str(pk)}' not found")

    async def _list_all(self) -> QueryRowResults:
        return copy.deepcopy(list(self._internal_storage))

    async def _run_query(self, query_name: str, **query_params) -> QueryResult:
        if self._has_custom_queries(query_name):
            return self._run_custom_queries(query_name, **query_params)
        if "list_all" == query_name:
            return copy.deepcopy(list(self._internal_storage))
        if "get_by_" in query_name:
            return self._get_by_attrs(**query_params)
        if "list_by_" in query_name:
            return self._list_by_attrs(**query_params)
        raise RuntimeError(f"Query {query_name} not found")

    def reset(self, initial_data: List[Dict] | None = None):
        if initial_data is None:
            self._internal_storage = []
            return None
        self._internal_storage = [
            self._prepare_to_save(r) for r in copy.deepcopy(initial_data)
        ]

    @property
    def _queries(self) -> Dict[QueryName, QueryCallable]:
        return {}

    def _has_custom_queries(self, query_name: str) -> bool:
        return query_name in self._queries

    def _run_custom_queries(self, query_name: str, **query_params):
        return self._queries[query_name](**query_params)

    def _searchable_internal_storage(self) -> List:
        if self._internal_storage:
            return self._prepare_row_results(
                [copy.deepcopy(r) for r in self._internal_storage]
            )
        return self._internal_storage  # pragma: no cover

    def _get_by_attrs(
        self, filtering_callable: Callable = _has_filters_match, **query_params
    ):
        for idx, e in enumerate(self._searchable_internal_storage()):
            if filtering_callable(e, query_params):
                return self._internal_storage[idx]
        return None

    def _list_by_attrs(
        self, filtering_callable: Callable = _has_filters_match, **query_params
    ):
        result = []
        for idx, e in enumerate(self._searchable_internal_storage()):
            if filtering_callable(e, query_params):
                result.append(self._internal_storage[idx])
        return result

    def _search_idx_from_data(self, data: Dict):
        pk = self._get_pk_from_data(data)
        return self._search_idx_from_pk(pk)

    def _search_idx_from_persistence_id(self, persistence_id: Id):
        pk = self._get_pk_from_persistence_id(persistence_id)
        return self._search_idx_from_pk(pk)

    def _search_idx_from_pk(self, pk: Dict[IdName, IdValue]):
        for idx, r in enumerate(self._internal_storage):
            if self._row_has_pk(r, pk):
                return idx
        return None

    @staticmethod
    def _row_has_pk(r, id):
        for k, v in id.items():
            if r.get(k) != v:
                return False
        return True

    @staticmethod
    def _pk_str(pk: Dict[IdName, IdValue]) -> str:
        ids_txt = [f"{k}={v}" for k, v in pk.items()]
        return ", ".join(ids_txt)


class SerializedDictPersistence(SerializedPersistence, DictPersistence):
    def __init__(self, initial_data: List[Dict] | None = None):
        SerializedPersistence.__init__(self, JsonService())
        DictPersistence.__init__(self, initial_data)



from abc import ABC
from typing import TypeVar

from griff.domain.common_types import Entity
from griff.infra.persistence.dict_persistence import DictPersistence
from griff.infra.repository.base_repository import BaseRepository

A = TypeVar("A", bound=Entity)


class FakeRepositoryMixin:  # pragma: no cover
    _persistence: DictPersistence

    def reset(self):
        self._persistence.reset()


class Repository(BaseRepository[A], ABC):
    async def get_by_id(self, entity_id: str) -> A | None:
        return await self._check_by_id(entity_id=entity_id)

    async def list_all(self) -> list[A]:
        return await self._list_all()

    async def run_query(self, query_name: str, **query_params):
        return await self._run_query(query_name=query_name, **query_params)

    async def check_by_aggregate(self, aggregate: A) -> A | None:
        return await self.get_by_id(entity_id=aggregate.entity_id)

    async def _get_persistence_by_aggregate(self, aggregate: A) -> dict | None:
        return await self._get_persistence_by_id(entity_id=aggregate.entity_id)



from typing import Type

import pytest
import typer
from griff.appli.event.event_handler import FakeEventHandler
from injector import singleton
from griff.runtime.factories.abstract_runtime_factory import Injectables
from griff.test_utils.testcases.cli_testcase import CliTestCase
from griff.utils.async_utils import AsyncUtils

from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_repository import {{cookiecutter.aggregate | pascal_case}}Repository
from {{ cookiecutter.context | snake_case }}._common.test_utils.{{ cookiecutter.context | snake_case }}_dtf import {{ cookiecutter.context | pascal_case }}Dtf
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_cli import {{cookiecutter.command | pascal_case}}CliController
from {{ cookiecutter.context | snake_case }}.entry_point import {{ cookiecutter.context | pascal_case }}EntryPoint
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_event import {{ cookiecutter.CommandEvent }}

@singleton
class FakeOn{{ cookiecutter.CommandEvent }}Handler(FakeEventHandler):
    on_event_type = {{ cookiecutter.CommandEvent }}

class Test{{ cookiecutter.command | pascal_case }}Cli(CliTestCase):
    @classmethod
    def entry_point_class(cls) -> Type[{{ cookiecutter.context | pascal_case }}EntryPoint]:
        return {{ cookiecutter.context | pascal_case }}EntryPoint

    @classmethod
    def with_injectables(cls) -> Injectables | None:
        return {}

    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.dtf = cls.get_injected({{cookiecutter.context | pascal_case}}Dtf)
        cls.dtf.reset_sequence_id(99990)
        cls.repository = cls.get_injected({{cookiecutter.aggregate | pascal_case}}Repository)
        cls.event_handler = cls.get_injected(FakeOn{{ cookiecutter.CommandEvent }}Handler)
        cls.existing_{{ cookiecutter.aggregate | snake_case }} = cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}()

    def setup_method(self):
        super().setup_method()
        self.controller = self.get_injected({{cookiecutter.command | pascal_case}}CliController)
        self.dtf.reset(1)

    async def async_setup(self):
        await super().async_setup()
        self.repository.reset()
        await self.dtf.persist_created("{{cookiecutter.aggregate | snake_case}}")

    """
    action
    """

    def test_{{ cookiecutter.command | snake_case }}_avec_un_pb_a_definir_echoue(self, capsys):
        match = "Echec action : Une erreur"
        with pytest.raises(typer.Exit):
            self.controller.action(
                **self.existing_{{ cookiecutter.aggregate | snake_case }}.model_dump(exclude={"entity_id"})
            )
        assert capsys.readouterr().out == f"{match}\n"

    def test_{{ cookiecutter.command | snake_case }}_reussi(self):
        assert self.controller.action(**self.existing_{{ cookiecutter.aggregate | snake_case }}.model_dump(exclude={"entity_id"})) is None
        self.assert_equals_resultset(
            {
                "db": AsyncUtils.async_to_sync(self.repository.run_query, "list_all"),
                "handled_events": self.event_handler.list_events_handled(),
            }
        )



from fastapi import status, Depends
from fastapi.security import OAuth2PasswordBearer
from griff.appli.command.command_bus import CommandBus
from griff.infra.api.api_controller import ApiController
from griff.infra.api.register_endpoint import register_endpoint
from injector import inject

from {{ cookiecutter.context | snake_case }}._common.api.{{ cookiecutter.context | snake_case }}_schemas import {{ cookiecutter.command | pascal_case }}In, {{ cookiecutter.command | pascal_case }}Out
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_command import {{ cookiecutter.command | pascal_case }}Command

oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/{{ cookiecutter.context | snake_case }}/{{ cookiecutter.command | snake_case }}")


class {{ cookiecutter.command | pascal_case }}Controller(ApiController):
    base_route = "/"

    @inject
    def __init__(self, command_bus: CommandBus):
        super().__init__()
        self._command_bus = command_bus

    @register_endpoint(
        route="/{{ cookiecutter.command | snake_case }}", method="POST", success_code=status.HTTP_200_OK
    )
    async def {{ cookiecutter.command | snake_case }}(self, payload: {{ cookiecutter.command | pascal_case }}In, token: str = Depends(oauth2_scheme)) -> {{ cookiecutter.command | pascal_case }}Out:
        command = {{ cookiecutter.command | pascal_case }}Command(**payload.model_dump())
        response = await self._command_bus.dispatch(command, context=self._get_token_context(token))
        return self.prepare_response(response)



from griff.test_utils.testcases.domain_testcase import DomainTestCase
from {{ cookiecutter.context | snake_case }}._common.test_utils.{{ cookiecutter.context | snake_case }}_dtf import {{ cookiecutter.context | pascal_case }}Dtf
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_domain import {{ cookiecutter.command | snake_case }}, Action{{cookiecutter.aggregate | pascal_case}}

class Test{{ cookiecutter.command | pascal_case }}Domain(DomainTestCase):
    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.dtf = {{ cookiecutter.context | pascal_case }}Dtf(start_id=99990)
        {{ cookiecutter.aggregate | snake_case }} = cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}()
        cls.{{ cookiecutter.aggregate | snake_case }}_action = Action{{cookiecutter.aggregate | pascal_case}}(**{{ cookiecutter.aggregate | snake_case }}.model_dump())

    def setup_method(self):
        super().setup_method()

    """
    {{ cookiecutter.command | snake_case }}
    """

    def test_{{ cookiecutter.command | snake_case }}_reussi(self):
        actual = {{ cookiecutter.command | snake_case }}(self.{{ cookiecutter.aggregate | snake_case }}_action)
        self.assert_equals_resultset(actual.model_dump())



from typing import List, Any

from griff.infra.api.api_router import ApiRouter
from injector import inject


{{ cookiecutter.context | pascal_case }}Controller = Any


class {{ cookiecutter.context | pascal_case }}ApiRouter(ApiRouter):
    @inject
    def __init__(self) -> None:
        self.ctrls: List[{{ cookiecutter.context | pascal_case }}Controller] = []
        super().__init__()

    def _list_controllers(self) -> List[{{ cookiecutter.context | pascal_case }}Controller]:
        return self.ctrls

    @property
    def _route_prefix(self) -> str:
        return "/{{ cookiecutter.context | snake_case }}"

    @property
    def _route_tags(self) -> List[str]:
        return ["{{ cookiecutter.context | snake_case }}"]



import copy
import datetime
from typing import Any, Union, Annotated
from gettext import gettext as _
import arrow
from arrow import Arrow
from pydantic import GetCoreSchemaHandler, GetJsonSchemaHandler
from pydantic.json_schema import JsonSchemaValue
from pydantic_core import core_schema

DateTimeIn = Union[str, datetime.datetime, datetime.date, int, float, Arrow]


class DateTime:
    _parse_error = _("datetime invalid (YYYY-MM-DD HH:mm:ss.SSSSSS)")

    def __init__(self, datetime: DateTimeIn):
        self._value = (
            datetime if isinstance(datetime, Arrow) else self._check_is_valid(datetime)
        )
        self._timezone = "UTC"

    def to_mysql_date(self) -> str:
        return self._format("YYYY-MM-DD")

    def to_mysql_datetime(self) -> str:
        return self._format("YYYY-MM-DD HH:mm:ss.SSSSSS")

    def to_date(self) -> datetime.date:
        return self._value.date()

    def to_datetime(self) -> datetime.datetime:
        return self._value.to(self._timezone).datetime

    def to_timestamp(self) -> float:
        return self._value.timestamp()

    def copy(self):
        return copy.copy(self)

    def format(self, format: str):
        return self._format(format)

    @classmethod
    def _check_is_valid(cls, value: Any) -> Arrow:
        try:
            return arrow.get(value)
        except (TypeError, arrow.parser.ParserError):
            raise ValueError(_(cls._parse_error))

    def __copy__(self):
        return DateTime(self._value)

    def __repr__(self):
        return self._value.format("YYYY-MM-DDTHH:mm:ss.SSSSSSZZ")

    def __eq__(self, other):
        return self._value == other._value

    def __ne__(self, other):
        return self._value != other._value

    def __ge__(self, other):
        return self._value >= other._value

    def __le__(self, other):
        return self._value <= other._value

    def __gt__(self, other):
        return self._value > other._value

    def __lt__(self, other):
        return self._value < other._value

    def _format(self, date_format) -> str:
        return self._value.to(self._timezone).format(date_format)

    def shift(self, shifting_amount: dict) -> "DateTime":
        """
        Shift a date with the amount

        Args:
            d (str|Arrow|datetime.datetime|datetime.date): arrow date to shift from
            shifting_amount (dict):  shifting amount (use {years: 1, months:-1})
        Returns:
            DateTime : shifted date
        """
        return DateTime(self._value.shift(**shifting_amount))


class _DateTimeTypePydanticAnnotation:
    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        _source_type: Any,
        _handler: GetCoreSchemaHandler,
    ) -> core_schema.CoreSchema:
        def validate_from_in(value: DateTimeIn | DateTime) -> DateTime:
            if isinstance(value, DateTime):
                return value
            result = DateTime(value)
            return result

        from_in_schema = core_schema.no_info_plain_validator_function(validate_from_in)

        return core_schema.json_or_python_schema(
            json_schema=from_in_schema,
            python_schema=core_schema.union_schema(
                [
                    # check if it's an instance first before doing any further work
                    from_in_schema,
                ]
            ),
            serialization=core_schema.plain_serializer_function_ser_schema(
                lambda instance: instance.to_mysql_datetime()
            ),
        )

    @classmethod
    def __get_pydantic_json_schema__(
        cls, _core_schema: core_schema.CoreSchema, handler: GetJsonSchemaHandler
    ) -> JsonSchemaValue:
        # Use the same schema that would be used for `int`
        return handler(
            core_schema.union_schema(
                [
                    core_schema.str_schema(),
                    core_schema.date_schema(),
                    core_schema.datetime_schema(),
                    core_schema.int_schema(),
                ]
            )
        )


DateTimeType = Annotated[DateTime, _DateTimeTypePydanticAnnotation]


class Date(DateTime):
    _parse_error = _("date invalid (YYYY-MM-DD)")

    def __init__(self, datetime: DateTimeIn):
        super().__init__(datetime)
        self._value = arrow.get(self._value.date())

    def __copy__(self):
        return Date(self._value)

    def __repr__(self):
        return self._value.format("YYYY-MM-DD")  # pragma: no cover

    def shift(self, shifting_amount: dict) -> "Date":
        return Date(self._value.shift(**shifting_amount))


class _DateTypePydanticAnnotation:
    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        _source_type: Any,
        _handler: GetCoreSchemaHandler,
    ) -> core_schema.CoreSchema:
        def validate_from_in(value: DateTimeIn | Date) -> Date:
            if isinstance(value, Date):
                return value
            result = Date(value)
            return result

        from_in_schema = core_schema.no_info_plain_validator_function(validate_from_in)

        return core_schema.json_or_python_schema(
            json_schema=from_in_schema,
            python_schema=core_schema.union_schema(
                [
                    # check if it's an instance first before doing any further work
                    from_in_schema,
                ]
            ),
            serialization=core_schema.plain_serializer_function_ser_schema(
                lambda instance: instance.to_mysql_date()
            ),
        )

    @classmethod
    def __get_pydantic_json_schema__(
        cls, _core_schema: core_schema.CoreSchema, handler: GetJsonSchemaHandler
    ) -> JsonSchemaValue:
        # Use the same schema that would be used for `int`
        return handler(
            core_schema.union_schema(
                [
                    core_schema.str_schema(),
                    core_schema.date_schema(),
                    core_schema.datetime_schema(),
                    core_schema.int_schema(),
                ]
            )
        )


DateType = Annotated[Date, _DateTypePydanticAnnotation]



from typing import TypeAlias, Type

from griff.appli.policies.policy import PolicyException
from griff.appli.command.command import Command
from griff.appli.command.command_handler import (
    CommandHandler,
    CommandErrorResponse,
    CommandSuccessResponse,
)
from injector import inject
from shared_kernel.domain.permission import PermissionAdminRequired
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_domain import {{ cookiecutter.command | snake_case }}, Action{{cookiecutter.aggregate | pascal_case}}
from {{ cookiecutter.context | snake_case }}._common.domain.{{ cookiecutter.aggregate | snake_case }} import Base{{cookiecutter.aggregate | pascal_case}}, {{cookiecutter.aggregate | pascal_case}}
from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_repository import {{cookiecutter.aggregate | pascal_case}}Repository
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_event import {{ cookiecutter.CommandEvent }}

from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_policies import {{cookiecutter.aggregate | pascal_case}}VerifieUnTrucPolicy, {{cookiecutter.aggregate | pascal_case}}VerifieUnTrucError


class {{ cookiecutter.command | pascal_case }}Command(Action{{cookiecutter.aggregate | pascal_case}}, PermissionAdminRequired, Command):
    ...

class {{ cookiecutter.command | pascal_case }}SuccessResponse(CommandSuccessResponse):
    content: {{cookiecutter.aggregate | pascal_case}}


{{ cookiecutter.command | pascal_case }}Error = {{cookiecutter.aggregate | pascal_case}}VerifieUnTrucError


class {{ cookiecutter.command | pascal_case }}ErrorResponse(CommandErrorResponse):
    content: {{ cookiecutter.command | pascal_case }}Error


{{ cookiecutter.command | pascal_case }}Response: TypeAlias = (
    {{ cookiecutter.command | pascal_case }}SuccessResponse | {{ cookiecutter.command | pascal_case }}ErrorResponse
)


class {{ cookiecutter.command | pascal_case }}Handler(
    CommandHandler[{{ cookiecutter.command | pascal_case }}Command, {{ cookiecutter.command | pascal_case }}Response]
):
    @inject
    def __init__(
        self, {{ cookiecutter.aggregate | snake_case }}_repository: {{cookiecutter.aggregate | pascal_case}}Repository, {{ cookiecutter.aggregate | snake_case }}_verifie_un_truc_policy: {{cookiecutter.aggregate | pascal_case}}VerifieUnTrucPolicy
    ):
        super().__init__()
        self.repository = {{ cookiecutter.aggregate | snake_case }}_repository
        self.{{ cookiecutter.aggregate | snake_case }}_verifie_un_truc_policy = {{ cookiecutter.aggregate | snake_case }}_verifie_un_truc_policy

    async def handle(self, command: {{ cookiecutter.command | pascal_case }}Command) -> {{ cookiecutter.command | pascal_case }}Response:
        try:
            await self.{{ cookiecutter.aggregate | snake_case }}_verifie_un_truc_policy.check()
        except PolicyException as policy_exception:
            return {{ cookiecutter.command | pascal_case }}ErrorResponse(content=policy_exception.error)

        {{ cookiecutter.aggregate | snake_case }} = {{ cookiecutter.command | snake_case }}(Action{{cookiecutter.aggregate | pascal_case}}(**command.model_dump()))
        await self.repository.save({{ cookiecutter.aggregate | snake_case }})
        event = {{ cookiecutter.CommandEvent }}(payload={{ cookiecutter.aggregate | snake_case }})
        return {{ cookiecutter.command | pascal_case }}SuccessResponse(content={{ cookiecutter.aggregate | snake_case }}, events=[event])

    @classmethod
    def listen_to(cls) -> Type[{{ cookiecutter.command | pascal_case }}Command]:
        return {{ cookiecutter.command | pascal_case }}Command



from typing import Optional

from injector import singleton

from griff.services.uniqid.generator.uniqid_generator import UniqIdGenerator


@singleton
class FakeUniqIdGenerator(UniqIdGenerator):
    def __init__(self, start_id=1):
        self._start = 0
        self._start = start_id - 1
        self._session = {"default": self._start}

    def next_id(self, name: Optional[str] = None) -> str:
        if name is None:
            name = "default"
        if name not in self._session:
            name_length = len(name)
            start_length = len(str(self._start))
            total_length = 10 + name_length + start_length
            if total_length > 26:
                raise ValueError(
                    f"Uniq Id will be too long {total_length} > 26, reduce start_id "
                    f"({self._start}={start_length}) or name ({name}={name_length})"
                )
            self._session[name] = self._start

        self._session[name] += 1
        return self._format_id(name)

    def reset(self, start_id: int = 1):
        self._start = start_id - 1
        self._session = {"default": self._start}

    def _format_id(self, name: str) -> str:
        if name == "default":
            return f"FAKEULID{self._session[name]:>018}"

        postfix = name.replace("_", "").upper()
        pad = 16 - len(postfix)
        id = f"FAKEULID-{postfix}-{self._session[name]:>0{pad}}"
        # noinspection Assert
        assert len(id) == 26, f"id length must be 26, got {len(id)}"
        return id



from injector import inject

from griff.appli.command.command import Command
from griff.appli.command.command_handler import CommandResponse
from griff.appli.command.command_middleware import CommandMiddleware
from griff.appli.event.event_bus import EventBus
from griff.appli.message.message_middleware import MessageContext


class CommandEventDispatchMiddleware(CommandMiddleware):
    @inject
    def __init__(self, event_bus: EventBus):
        super().__init__()
        self._event_bus = event_bus

    async def dispatch(
        self, message: Command, context: MessageContext | None = None
    ) -> CommandResponse:
        response = await self._next_dispatch(message, context)
        if response.is_success:
            for event in response.events:
                await self._event_bus.dispatch(event, context)
        return response



import random
from abc import ABC
from decimal import Decimal
from typing import Self, List, Dict, Optional, Any

from griff.domain.common_types import Entity
from griff.infra.persistence.persistence import Persistence
from griff.infra.repository.repository import Repository
from griff.services.date.fake_date_service import FakeDateService
from griff.services.service_locator.service_locator import ServiceLocator
from griff.services.uniqid.uniqid_service import UniqIdService


class DataTestFactory(ABC):
    def __init__(self, start_id: int = 1):
        self.uniqid_service = ServiceLocator.get(UniqIdService)
        self.date_service = FakeDateService()
        self._repositories: Dict[str, Repository] = {}
        self._sequence: Dict = {}
        self._created: Dict = {}
        random.seed(42)
        self._current_random_state = random.getstate()
        self._random_states: dict[str, Any] = {}
        self.reset(start_id)

    async def persist(
        self, repository: Repository, data: list[Entity] | Entity
    ) -> Self:
        if isinstance(data, list) is False:
            await repository.save(data)
            return self

        for a in data:
            await repository.save(a)
        return self

    async def persist_by_persistence(
        self, persistence: Persistence, data: List[Entity] | Entity
    ) -> Self:
        initial_data = data if isinstance(data, list) else [data]
        persistence.reset()
        for d in initial_data:
            await persistence.insert(d.model_dump())
        return self

    def random_decimal(self, min, max, for_name: str | None = None) -> Decimal:
        self._random_setstate(for_name)
        min = float(min) * 100
        max = float(max) * 100
        val = Decimal(random.uniform(min, max)) / 100
        self._save_current_random_state(for_name)
        return val

    def random_int(self, min, max, for_name: str | None = None) -> int:
        self._random_setstate(for_name)
        val = random.randrange(min, max)
        self._save_current_random_state(for_name)
        return val

    def random_existing_asset(
        self, asset_type: str, for_name: str | None = None
    ) -> Any:
        self._random_setstate(for_name)
        val = random.choice(self._created[asset_type])
        self._save_current_random_state(for_name)
        return val

    def next_sequence(self, name: str) -> int:
        if name not in self._sequence:
            self._sequence[name] = 0
        self._sequence[name] += 1
        return self._sequence[name]

    def current_sequence(self, name: str) -> int:
        return self._sequence[name]

    def list_created(self, name):
        return self._created[name] if name in self._created else []

    def reset_sequence_id(self, start: int = 1) -> Self:
        self.uniqid_service.reset(start)
        return self

    def reset(self, start_id=1) -> Self:
        self.reset_sequence_id(start_id)
        for repository in self._repositories.values():
            if repository:
                repository.reset()
        return self

    async def persist_created(
        self, name: str, data: Optional[list[Entity] | Entity] = None
    ) -> Self:
        if self._has_repository(name) is False:
            raise RuntimeError(f"Repository {name} not found")
        data = data if data else self._created[name]
        return await self.persist(self._repositories[name], data)

    async def persist_all_created(self):
        """
        !! les données seront persistées dans l'ordre de création de chaque type
        de données créées
        """
        for name, created in self._created.items():
            if self._has_repository(name) is False:
                continue
            await self.persist_created(name, created)
        return self

    def _add_created(self, name, created):
        if name not in self._created:
            self._created[name] = []
        self._created[name].append(created)
        return created

    def _has_repository(self, name: str) -> bool:
        return name in self._repositories and self._repositories[name] is not None

    def _random_setstate(self, name: str | None = None) -> None:
        if name and name in self._random_states:
            random_state = self._random_states[name]
        else:
            random_state = self._current_random_state
        random.setstate(random_state)

    def _save_current_random_state(self, name: str | None = None) -> None:
        if name:
            self._random_states[name] = random.getstate()
            return None
        self._current_random_state = random.getstate()



from typing import Type

from griff.runtime.factories.abstract_runtime_factory import Injectables
from griff.test_utils.testcases.repository_testcase import RepositoryTestCase

from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_repository import (
    {{cookiecutter.aggregate | pascal_case}}Repository,
)
from {{ cookiecutter.context | snake_case }}._common.test_utils.{{ cookiecutter.context | snake_case }}_dtf import {{ cookiecutter.context | pascal_case }}Dtf
from {{ cookiecutter.context | snake_case }}.entry_point import {{ cookiecutter.context | pascal_case }}EntryPoint


class Test{{cookiecutter.aggregate | pascal_case}}Repository(RepositoryTestCase):
    @classmethod
    def entry_point_class(cls) -> Type[{{ cookiecutter.context | pascal_case }}EntryPoint]:
        return {{ cookiecutter.context | pascal_case }}EntryPoint

    @classmethod
    def with_injectables(cls) -> Injectables | None:
        return {}

    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.dtf = {{ cookiecutter.context | pascal_case }}Dtf()
        cls.repository = cls.get_injected({{cookiecutter.aggregate | pascal_case}}Repository)
        cls.{{ cookiecutter.aggregate | snake_case }}s = [
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
            cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}(),
        ]

    def setup_method(self):
        super().setup_method()

    """
    CRUD
    """

    async def test_crud_reussi(self) -> None:
        update_data = self.dtf.creer_{{ cookiecutter.aggregate | snake_case }}().model_dump(exclude={"entity_id"})
        self.assert_equals_resultset(
            await self.run_crud(
                repository=self.repository,
                aggregates=self.{{ cookiecutter.aggregate | snake_case }}s,
                aggregate_to_update=self.{{ cookiecutter.aggregate | snake_case }}s[1],
                update_data=update_data,
                aggregate_to_delete=self.{{ cookiecutter.aggregate | snake_case }}s[1],
            ),
        )



def j2_snake_case_filter(value):
    return value.strip().replace(" ", "_").lower()


def j2_kebab_case_filter(value):
    return j2_snake_case_filter(value).replace("_", "-")


def j2_camel_case_filter(value):
    words = j2_snake_case_filter(value).replace("_", " ").split()
    return words[0].lower() + "".join(word.capitalize() for word in words[1:])


def j2_pascal_case_filter(value):
    words = j2_snake_case_filter(value).replace("_", " ").split()
    return "".join(word.capitalize() for word in words)



import asyncio
import os
import random

from loguru import logger
from starlette.middleware.base import BaseHTTPMiddleware


class FakeLagTimeMiddleware(BaseHTTPMiddleware):  # pragma: no cover
    """
    simulate response lag
    """

    def __init__(self, app):
        super().__init__(app)
        self._durations = [0, 0.3, 0.5, 0.5, 0.5, 0.5, 0.6, 0.8, 0.8, 1]
        self._is_active = bool(os.getenv("API_FAKE_LAG_TIME", default=0))

    async def dispatch(self, request, call_next):
        # Proceed with the request
        response = await call_next(request)
        if self._is_active:
            await self.simulate_lagtime()
        return response

    async def simulate_lagtime(self):
        sleep_duration = random.choice(self._durations)
        if sleep_duration > 0:
            logger.info(f"Fake lag time of {sleep_duration}s ...")
            await asyncio.sleep(sleep_duration)
            logger.info("Fake lag time over")
        else:
            logger.info("Fake lag ignored")



import pytest
from typing import Type

from griff.appli.event.event_bus import EventBus
from griff.appli.event.event_handler import FakeEventHandler
from griff.runtime.factories.abstract_runtime_factory import Injectables
from griff.test_utils.testcases.api_testcase import ApiTestCase, FastApiTestClient
from injector import singleton

from access._common.test_utils.mixins.access_test_mixin import AccessTestMixin
from {{ cookiecutter.context | snake_case }}.entry_point import {{ cookiecutter.context | pascal_case }}EntryPoint
from {{ cookiecutter.context | snake_case }}.{{ cookiecutter.command | snake_case }}.{{ cookiecutter.command | snake_case }}_event import {{ cookiecutter.CommandEvent }}
from {{ cookiecutter.context | snake_case }}._common.repositories.{{ cookiecutter.aggregate | snake_case }}_repository import {{cookiecutter.aggregate | pascal_case}}Repository
from {{ cookiecutter.context | snake_case }}._common.test_utils.{{ cookiecutter.context | snake_case }}_dtf import {{ cookiecutter.context | pascal_case }}Dtf
from {{ cookiecutter.context | snake_case }}._common.api.{{ cookiecutter.context | snake_case }}_schemas import {{ cookiecutter.command | pascal_case }}In

@singleton
class FakeOn{{ cookiecutter.CommandEvent }}Handler(FakeEventHandler):
    on_event_type = {{ cookiecutter.CommandEvent }}

@pytest.mark.skip(reason="not yet implemented")
class Test{{ cookiecutter.command | pascal_case }}Api(AccessTestMixin, ApiTestCase):
    @classmethod
    def entry_point_class(cls) -> Type[{{ cookiecutter.context | pascal_case }}EntryPoint]:
        return {{ cookiecutter.context | pascal_case }}EntryPoint

    @classmethod
    def with_injectables(cls) -> Injectables | None:
        return super().with_injectables()

    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.dtf = cls.get_injected({{cookiecutter.context | pascal_case}}Dtf)
        cls.dtf.reset_sequence_id(99990)
        cls.event_bus = cls.get_injected(EventBus)
        cls.event_handler = cls.get_injected(FakeOn{{ cookiecutter.CommandEvent }}Handler)
        cls.repository = cls.get_injected({{cookiecutter.aggregate | pascal_case}}Repository)
        {{ cookiecutter.aggregate | snake_case }} = cls.dtf.creer_{{ cookiecutter.aggregate | snake_case }}()
        cls.api_in = {{ cookiecutter.command | pascal_case }}In(**{{ cookiecutter.aggregate | snake_case }}.model_dump())
        # hack temporaire le temps de trouver comment se passer de AccessTestMixin
        FastApiTestClient.default_access_token = cls.get_access_token(
            cls.active_account
        )

    def setup_method(self):
        super().setup_method()
        self.dtf.reset(1)

    """
    POST /{{ cookiecutter.context | snake_case }}/{{ cookiecutter.command | snake_case }}
    """

    async def test_{{ cookiecutter.command | snake_case }}_avec_donnees_obligatoires_manquante_echoue(self):
        response = await self.client.post(self.reverse_url("{{ cookiecutter.command | snake_case }}"), json={})
        self.assert_response_equals_resultset(response)

    async def test_{{ cookiecutter.command | snake_case }}_reussi(self):
        response = await self.client.post(self.reverse_url("{{ cookiecutter.command | snake_case }}"), json=self.api_in.model_dump())
        self.assert_equals_resultset(
            await self.prepare_success_resultset(response, self.repository, self.event_handler)
        )

    async def test_{{ cookiecutter.command | snake_case }}_pour_chaque_permission(self):
        actual = await self.assert_pour_chaque_permission(
            method="post",
            url=self.reverse_url("{{ cookiecutter.command | snake_case }}"),
            json=self.api_in.model_dump(),
        )
        self.assert_equals_resultset(actual)



from abc import ABC
from typing import TypeAlias
from griff.domain.common_types import Aggregate, EntityId
from pydantic import BaseModel

{{cookiecutter.aggregate | pascal_case}}Id: TypeAlias = EntityId


class Base{{cookiecutter.aggregate | pascal_case}}(BaseModel, ABC):
    ...

class {{cookiecutter.aggregate | pascal_case}}(Base{{cookiecutter.aggregate | pascal_case}}, Aggregate):
    entity_id: {{cookiecutter.aggregate | pascal_case}}Id



from griff.appli.policies.policy import Policy, PolicyException
from griff.utils.errors import ConflictError

class {{cookiecutter.aggregate | pascal_case}}VerifieUnTrucError(ConflictError):
    ...

# à renommer
class {{cookiecutter.aggregate | pascal_case}}VerifieUnTrucPolicy(Policy):
    async def check(self, **kwargs) -> None:
        # some code
        if True:
            return None
        message = _("echec")
        raise PolicyException({{cookiecutter.aggregate | pascal_case}}VerifieUnTrucError(message=message))



from griff.infra.cli.cli_router import CliRouter
from injector import inject, singleton


@singleton
class {{ cookiecutter.context | pascal_case }}CliRouter(CliRouter):
    @inject
    def __init__(self):
        super().__init__()
        # self._app.add_typer(ctrl.get_app(), name=ctrl.get_command_name())

    def get_command_group_name(self) -> str:  # pragma: no cover
        return "{{ cookiecutter.context | snake_case }}"



from jinja2.ext import Extension

from griff.infra.cli.controllers.griff.jinja2.filters.j2_str_filters import (
    j2_snake_case_filter,
    j2_kebab_case_filter,
    j2_camel_case_filter,
    j2_pascal_case_filter,
)


class GriffExtension(Extension):
    def __init__(self, environment):
        super().__init__(environment)
        environment.filters["snake_case"] = j2_snake_case_filter
        environment.filters["kebab_case"] = j2_kebab_case_filter
        environment.filters["camel_case"] = j2_camel_case_filter
        environment.filters["pascal_case"] = j2_pascal_case_filter



from griff.infra.persistence.db_persistence import SerializedDbPersistence, DbPersistence
from griff.infra.persistence.dict_persistence import SerializedDictPersistence, DictPersistence
from injector import singleton

class {{ cookiecutter.aggregate | pascal_case }}Persistence(SerializedDbPersistence):
    def _get_relative_sql_queries_path(self) -> str:
        return "{{ cookiecutter.context| snake_case }}/_common/repositories/sql/{{ cookiecutter.aggregate | snake_case }}.sql"

@singleton
class {{ cookiecutter.aggregate | pascal_case }}FakePersistence(SerializedDictPersistence):
    ...

######
# OU (supprimer l'implémentation inutile)
######
class {{ cookiecutter.aggregate | pascal_case }}Persistence(DbPersistence):
    def _get_relative_sql_queries_path(self) -> str:
        return "{{ cookiecutter.context| snake_case }}/_common/repositories/sql/{{ cookiecutter.aggregate | snake_case }}.sql"

@singleton
class {{ cookiecutter.aggregate | pascal_case }}FakePersistence(DictPersistence):
    ...
